unit Code19124u1;

(*See const sPrgmName and sVers*)

(*===== This is...
A "skelton" which can be used as the basis for
a program to fetch a textfile from backing store,
encrypt it, and send the encrypted version back
to backing store.

To build it out past the original state, most of
the changes should go inside the sEncrypt function
implementaion. It may be necessary to add bits
elsewhere, if the encryption scheme uses, say,
a key.

The plaintext can also be typed into this app
directly, and even saved as plaintext from this
app.

To save various headaches, fancy hanling of
wordwrap and ends of lines has been dodged in
this simple scenario.
Lines do not wrap.
The "hidden codes" which cause a new line in
a simple text file will be present in the
encrypt in the same places as they appear in
the plaintext... creating a weakness in the
encryption, by the way.

*)

{$mode objfpc}{$H+}

interface

(*Good news!....
Textpad... my prefered text editor...
     puts 0D 0A at ends of lines.
Lazarus (memo).Lines.SaveToFile((filename))...
     puts 0D 0A at ends of lines.
Same EOL conventions! (I can use SaveToFile)

*)

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics,
  Dialogs, StdCtrls, IniFiles;

const sPrgmName='Code19124';
      sVers='25 Jan 19.. may have changes other than this from the published zip';

      kPlain=0;//To "say" "The plaintext"
      kEncry=1;//To "say" "The encrupt"...
      //Used in things like LoadMemo(kPlain)

      sIniFileName='Code19124ini.txt';

type

  { TCode19124f1 }

  TCode19124f1 = class(TForm)
    buLoadEncry: TButton;
    buSavePlain: TButton;
    buLoadPlain: TButton;
    buSaveEncry: TButton;
    buEncrypt: TButton;
    buQuit: TButton;
    buDecrypt: TButton;
    mePlain: TMemo;
    meEncry: TMemo;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure buDecryptClick(Sender: TObject);
    procedure buEncryptClick(Sender: TObject);
    procedure buLoadEncryClick(Sender: TObject);
    procedure buLoadPlainClick(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buSaveEncryClick(Sender: TObject);
    procedure buSavePlainClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    sTmp:string;
    meTmp:TMemo;
    bTmp:byte;
    chTmp:char;
    procedure LoadMemo(bWhich:byte);
    procedure SaveMemo(bWhich:byte);
    function sEncrypt(sSourceL:String):String;
    function sDecrypt(sSourceL:String):String;

  public
    { public declarations }
  end;

var
  Code19124f1: TCode19124f1;

implementation

{$R *.lfm}

{ TCode19124f1 }

procedure TCode19124f1.FormCreate(Sender: TObject);
var dfIniFile:TIniFile;//"df" in name from "datafile".
begin
  Code19124f1.caption:=sPrgmName+', version '+sVers;
  application.title:=sPrgmName;

  //Until something fanch is done about the CR/LF codes
  //in the text file, at least turn off Wordwrap...
  mePlain.WordWrap:=false;
  meEncry.WordWrap:=false;

//Next uses an ini file to re-open window in
//   previous state...
//See http://sheepdogguides.com/lut/lt2Nc.htm
//   for a tutorial on ini files. Be sure to
//   put IniFiles in the unit's uses if you
//   want to use ini files.

dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+sIniFileName);

with dfIniFile do begin
   Code19124f1.height:=ReadInteger('WindowProperties','Height',340);
   Code19124f1.width:=ReadInteger('WindowProperties','Width',480);
   Code19124f1.top:=ReadInteger('WindowProperties','Top',10);
   Code19124f1.left:=ReadInteger('WindowProperties','Left',10);
   end;//with...

dfIniFile.Free;
//End "... using an ini file..."
end;//FormCreate

procedure TCode19124f1.FormClose(Sender: TObject; var CloseAction: TCloseAction
  );
var dfIniFile:TInifile;
begin
//Next uses an ini file to save window's
//    properties, so it can open next time
//    as it is now...

dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+sIniFileName);
 with dfIniFile do begin

  WriteInteger('WindowProperties','Height',Code19124f1.height);
  WriteInteger('WindowProperties','Width',Code19124f1.width);
  WriteInteger('WindowProperties','Top',Code19124f1.top);
  WriteInteger('WindowProperties','Left',Code19124f1.left);

  end;//with...
dfIniFile.Free;
//end of saving data to ini file.


end;//FormClose



procedure TCode19124f1.buQuitClick(Sender: TObject);
begin
  close;//application.terminate; does NEARLY the same thing,
  //but failes to call the OnFormClose handler... a problem for the
  //ini file system built into this.
end;

procedure TCode19124f1.buSaveEncryClick(Sender: TObject);
begin
  SaveMemo(kEncry);
end;

procedure TCode19124f1.buLoadEncryClick(Sender: TObject);
begin
  LoadMemo(kEncry);
end;

procedure TCode19124f1.buSavePlainClick(Sender: TObject);
begin
  SaveMemo(kPlain);
end;

procedure TCode19124f1.buLoadPlainClick(Sender: TObject);
begin
  LoadMemo(kPlain);
end;

procedure TCode19124f1.LoadMemo(bWhich:byte);
begin
if bWhich=kPlain then meTmp:=mePlain//no ; here
  else meTmp:=meEncry;

if OpenDialog1.Execute then
begin
  sTmp:=OpenDialog1.Filename;
  end;//if OpenDialog1

if FileExists(sTmp) then begin
  meTmp.Lines.LoadFromFile(sTmp);
  end//no ; here
 else begin
   showmessage('There is no file with that '+
   'name in that directory');
 end;//of elese
end;//LoadMemo

procedure TCode19124f1.SaveMemo(bWhich:byte);
begin
if bWhich=kPlain then meTmp:=mePlain//no ; here
  else meTmp:=meEncry;

if SaveDialog1.Execute then begin
 try
 meTmp.Lines.SaveToFile(SaveDialog1.Filename);
 except
   on EStreamError do begin
   showmessage('Could not save.');
   end;//On eStreamError
 end;//Try.. except
 end//no ; here... end of if.. then...
else begin
 showmessage('Did not save.');
 end;
end;//SaveMemo

procedure TCode19124f1.buEncryptClick(Sender: TObject);
var iDoingRow,iRows:integer;
  sSource,sDest{tination}:String;
(*Beware "fencepost" issues,
    issues arising from counting from zero,
    and "empty line" issues!

  If mePlain has 1/ 2/ 3 in it, where "/" signifies
    that the user has pressed enter to start a new
    line. Note: The user has NOT pressed enter again
    after the "3". The cursor will still be on
    the line with the 3.

    ... then, to encrypt those lines, you work with
         mePlain.Lines[0],
         mePlain.Lines[1], and
         mePlain.Lines[2]

    mePlain.Lines.count will return 3.

    The first line in mePlain is mePlain.Lines[0]
    The last, if there are THREE lines, is
       mePlain.Lines[2]

  BUT! If the user has entered 1/ 2/ 3/....
     In other words, what was entered previously,
     EXCEPT *in addition* the user HAS pressed
     enter after the 3. The cursor is on the
     line following the 3, but there is not
     yet anything on that line.

     In this case, mePlain.Lines.count will
     return 4... there are three lines with
     something on them, AND "a 4th line", even
     though that 4th line is empty.

  Don't worry... as long as you provide for
     skipping over "empty" lines, as you need
     to anyway, an empty line at the end of
     the file is no big deal. However, you
     should write an empty line to meEncry
     each time you see one in mePlain, to
     "inject" the right number of CR/LFs into
     the file.

  (Of course, if the program is doing fancy
     things to hide, in the encrypted version,
     where CR/LFs came in the plaintext, this
     all becomes more complicated!)

*)

begin
  iRows:=mePlain.Lines.count;
  meEncry.Lines.clear;
  for iDoingRow:=0 to iRows-1 do begin

    sSource:=mePlain.Lines[iDoingRow];

    sDest:=sEncrypt(sSource);

    meEncry.Lines[iDoingRow]:=sDest;
    end;//for
end;//buEncryptClick

procedure TCode19124f1.buDecryptClick(Sender: TObject);
//(See notes on buEncrypt click... this is just inverse
var iDoingRow,iRows:integer;
  sSource,sDest{tination}:String;
begin
  iRows:=meEncry.Lines.count;
  mePlain.Lines.clear;
  for iDoingRow:=0 to iRows-1 do begin

    sSource:=meEncry.Lines[iDoingRow];

    sDest:=sDecrypt(sSource);

    mePlain.Lines[iDoingRow]:=sDest;
    end;//for
end; //buDecryptClick

function TCode19124f1.sEncrypt(sSourceL:String):String;
(*HERE IS THE FIRST of 2 BITs WHERE YOU MAKE CHANGES
  Very little needs changing elsewhere,
    although it would be right to change the values
    in the sVers and sPrgmName constants, and perhaps
    add rems at the top of the code. *)

var iDoingCol:integer;
  sResult:String;
begin
(*
If you want to have a simple life, and want
to be able to look at the encrypt in a memo,
or in a simple text editor, ensure that the
only Ascii numbers stored in the encrypt fall
**between** 31 and 127... i.e. excluding those
two numbers, and all below and above them.

Under those rules, "tabs" cannot be saved in
the encrypt, by the way. (ASCII 9).

You may find the ord() function useful;
ord('A'); gives you the Ascii code for the
character 'A' (It happens to be 65)

The chr() function will also be useful...
chr(65) returns the character 'A'

When you save the memo with the encrypted text,
the file will have some CR/LF characters in
in it, but it should only have the ones put in
"behind the sceens by "the system" to record
where the lines ended in the memo.

--------
You will probably want to work your
way through the contents of sTmp one
character at a time... the following
would put them on the screen one after
the other. Note that for this, contrary
to te way the lines (rows) were
numbered, what you and I would call
"the first" is at position "1".

The following gives you a very crude Caesar code
  ...*)

sResult:='';//initialize as empty.

for iDoingCol:=1 to length(sSourceL) do begin
    //Go through the line, one character at a time...
    bTmp:=ord(sSourceL[iDoingCol]);//Turn current letter into a number
    if bTmp=126 then bTmp:=31;//to constrain result to 32 to 126
    chTmp:=chr(bTmp+1);//turn number back into a character
    sResult:=sResult+chTmp;
    end;//for..

result:=sResult;
end;//sEncrypt

function TCode19124f1.sDecrypt(sSourceL:String):String;
(*HERE IS THE SECOND of 2 BITs WHERE YOU MAKE CHANGES
  Very little needs changing elsewhere,
    although it would be right to change the values
    in the sVers and sPrgmName8 constants, and perhaps
    add rems at the top of the code.

  See also the notes in function sEncrypt*)

var iDoingCol:integer;
  sResult:String;
begin
sResult:='';//initialize as empty.

for iDoingCol:=1 to length(sSourceL) do begin
  //Go through the line, one character at a time...
  bTmp:=ord(sSourceL[iDoingCol]);//Turn current letter into a number
  if bTmp=32 then bTmp:=127;//to constrain result to 32 to 126
  chTmp:=chr(bTmp-1);//turn number back into a character
  sResult:=sResult+chTmp;
  end;//for..

result:=sResult;
end;//sDecry

end.
